#pragma once

#include <cstdint>
#include <linux/videodev2.h>
#include <string>
#include <vector>

//! Small C++ wrapper around V4L2 device file descriptor
class V4L2Device {
public:
    //! Device buffer wrapper
    struct Buffer {
        //! Driver buffer index
        int index;
        //! Buffer mmaped data
        uint8_t* data;
        //! Buffer size
        size_t length;
    };

    //! Create and open V4L2 device from path (/dev/videoX or /dev/vbiX)
    V4L2Device(std::string& path);
    virtual ~V4L2Device();

    //! Check device is open
    bool isOpen() const;
    //! There was error during last operation
    bool isError() const;
    //! Error code
    int error() const;
    //! Error description
    std::string errorString() const;
    //! File handle
    int fd() const;

    //! Device type
    v4l2_buf_type type() const;
    //! Device name
    std::string name() const;

    //! Call ioctl on device
    int ioctl(int ctl, void* param);

    //! Request buffers from driver
    std::vector<Buffer> requestBuffers(int count);
    //! Free all allocated buffers
    void freeBuffers(std::vector<Buffer>& buffers);

private:
    V4L2Device(V4L2Device const&) = delete;
    V4L2Device& operator=(V4L2Device const&) = delete;

    int m_fd = -1;
    v4l2_capability m_caps;
    int m_lastError = 0;
};
